#include <opencv2/opencv.hpp>
#include <opencv2/tracking.hpp>

class Tracker {
private:

    cv::Ptr<cv::Tracker> tracker;
    cv::Rect rect;

public:

    Tracker(cv::Mat &frame, cv::Rect rect_) {
        rect = rect_;
        // target processing
        tracker = cv::TrackerMIL::create();
        // tracker = cv::TrackerKCF::create();
        tracker->init(frame, rect);
    }

    cv::Rect detect(cv::Mat &frame) {
        // frame processing
        tracker->update(frame, rect);
        return rect;
    }

};

int main()
{
    // selection of the followed object
    static cv::Point refPt0, refPt1;
    static cv::Point selPt0, selPt1;
    static enum { STARTING, SELECTING, SELECTED } selection = STARTING;
    cv::MouseCallback mouseHandler = [](int event, int x, int y, int flags, void* userdata) {
        if (event == cv::EVENT_LBUTTONDOWN) {
            refPt0 = cv::Point(x, y);
            selection = SELECTING;
        }
        else if (event == cv::EVENT_LBUTTONUP) {
            selPt0 = refPt0;
            selPt1 = cv::Point(x, y);
            selection = SELECTED;
        }
        else if (selection == SELECTING) {
            refPt1 = cv::Point(x, y);
        }
    };

    cv::namedWindow("image");
    cv::setMouseCallback("image", mouseHandler);

    cv::VideoCapture camera(0, cv::CAP_DSHOW);
    cv::Ptr<Tracker> tracker = nullptr;
    for (;;) {
        cv::Mat frame;
        camera >> frame;
        if (frame.empty()) break;
        cv::flip(frame, frame, 1); // mirror

        cv::Mat res = frame.clone();
        if (selection == SELECTED) {
            cv::Point tl(std::min(selPt0.x, selPt1.x), std::min(selPt0.y, selPt1.y));
            cv::Point br(std::max(selPt0.x, selPt1.x), std::max(selPt0.y, selPt1.y));
            printf("[%d,%d] - [%d,%d]\n", tl.x, tl.y, br.x, br.y);
            if (tl.x < br.x && tl.y < br.y) {
                cv::Rect rect(tl, br);
                cv::imshow("target", frame(rect));
                tracker = new Tracker(frame, rect);
            }
            selection = STARTING;
        }
        else if (selection == SELECTING) {
            cv::rectangle(res, cv::Point(std::min(refPt0.x, refPt1.x), std::min(refPt0.y, refPt1.y)), cv::Point(std::max(refPt0.x, refPt1.x), std::max(refPt0.y, refPt1.y)), cv::Scalar(0, 255, 0), 1);
        }
        else if (selection == SELECTED) {
            cv::rectangle(res, cv::Point(std::min(selPt0.x, selPt1.x), std::min(selPt0.y, selPt1.y)), cv::Point(std::max(selPt0.x, selPt1.x), std::max(selPt0.y, selPt1.y)), cv::Scalar(0, 255, 127), 1);
        }

        if (tracker) {
            cv::Rect rect = tracker->detect(frame);
            printf("%d, %d, %d, %d\n", rect.x, rect.y, rect.width, rect.height);
            cv::rectangle(res, rect, cv::Scalar(0, 255, 0), 2);
        }

        cv::imshow("image", res);
        if (cv::waitKey(1) == 27) break;
    }

    camera.release();
    cv::destroyAllWindows();
    return 0;
}
